#include <math.h>

#define CONST31 2147483647 // 2^31-1

static double fastRand(void) // returns double in the range 0.0 to 1.0 
{
	static unsigned long x = 1234567897UL; // initial seed value
	x = 69069*x + 362437;
	return (x & CONST31)/((double)CONST31);
}




__declspec(dllexport)

void __cdecl MB_RotateYX( int nPts, int * listPtrs, 
				  const int x_0, const int y_0, const int z_0,
               	  const int viewAngle, const int tiltAngle, const int scale, const int z_rho, 
				  const int scrXOffs, const int scrYOffs,
				  int const minScrX, int const maxScrX, int const minScrY, int const maxScrY, 
				  int * nXformedPts )
{

	// x_0, y_0, z_0 are reals multiplied by 65536
	// viewAngle, titleAngle are reals (in radians) multiplied by 65536
	
	int i, n;
	
	double sA, cA, sB, cB;
    double x, y, z, dx, dy, dz;
	double x1, y1, z1, x2, y2, z2;
	double k, scrX, scrY;
	
	double x0 = x_0 / 65536.0;
	double y0 = y_0 / 65536.0;
	double z0 = z_0 / 65536.0;
	double A = viewAngle / 65536.0;
	double B = tiltAngle / 65536.0;
	
	int *activeList;
	double *xList, *yList, *zList, *txList, *tyList, *tzList;
	unsigned long *indexList;
	
	activeList = (int *)listPtrs[0];
	xList = (double *)listPtrs[1]; 
	yList = (double *)listPtrs[2]; 
	zList = (double *)listPtrs[3]; 
	txList = (double *)listPtrs[4]; 
	tyList = (double *)listPtrs[5]; 
	tzList = (double *)listPtrs[6]; 
	indexList = (unsigned long *)listPtrs[7];
	
	sA = sin(A);
	cA = cos(A);
	sB = sin(B);
	cB = cos(B);
	
	n = 0; // no. of transformed points

	for (i = 0; i<nPts; i++)
	{
		
		if (activeList[i] == 0){ continue; } 
		
		x = xList[i];
		y = yList[i];
		z = zList[i];
		
		dx = x - x0;
		dy = y - y0;
		dz = z - z0;
		
		// rotate about Y axis
		x1 = dx*cA + dz*sA;
		y1 = dy;
		z1 = dz*cA - dx*sA;
		
		// rotate about X axis
		x2 = x1;
		y2 = y1*cB - z1*sB;
		z2 = y1*sB + z1*cB;
		
		z2 += z_rho;
		
		if (z2 > 1)
		{
			k = scale / z2;
			scrX = scrXOffs + k*x2;
			if (scrX>=minScrX && scrX<=maxScrX)
			{
				scrY = scrYOffs + (k*y2);
				if (scrY>=minScrY && scrY<=maxScrY)
				{
					txList[n] = scrX;
					tyList[n] = scrY;
					tzList[n] = z2;
					indexList[n] = i;
					n++;
				}
			}				
				
		}
	}

	*nXformedPts = n;
	
	return;
}




__declspec(dllexport)

void __cdecl MB_GetNearbyObjects( unsigned int objType, unsigned int minDist, unsigned int nObjs,
                          unsigned int *objIndexList, unsigned int *objTypeList,
                          double *x0, double *z0, double *objXList, double *objZList,
						  unsigned long *outList, unsigned int *nNearbyObjs )
{

	unsigned int i, j, d, n;
	double x, z, dx, dz;
	
	n = 0;
	d = minDist * minDist;
	x = *x0;
	z = *z0;
	
	for (i = 0; i < nObjs; i++){
		j = objIndexList[i];
		if (objTypeList[j] == objType){
			dx = x - objXList[j];
			dz = z - objZList[j];
			if (dx*dx + dz*dz <= d){
				outList[n] = j;
				n++;
			}
		}
	}
	
	*nNearbyObjs = n;
	
	return;
}




__declspec(dllexport)

void __cdecl MB_GetRandom3DVector( double *dx, double *dy, double *dz )
{

	double a, b, c, x, y, z, x1, y1, z1, x2, y2, z2, x3, y3, z3;
	double sA, cA, sB, cB, sC, cC;
	double pi2 = 2.0 * 3.14159265359;

	// get random rotation angles in the range 0 to 2pi
	a = pi2 * fastRand();
	b = pi2 * fastRand();
	c = pi2 * fastRand();
	
	// calc sines & cosines
	sA = sin(a);
	cA = cos(a);
	sB = sin(b);
	cB = cos(b);
	sC = sin(c);
	cC = cos(c);

	// initial (XYZ) vector to rotate (0.0, 1.0, 0.0)
	x = 0.0;
	y = 1.0;
	z = 0.0;
	
    // rotate about X-axis
	x1 = x;
	y1 = y*cA - z*sA;
	z1 = y*sA + z*cA;
	
	// rotate about Y-axis
	x2 = x1*cB + z1*sB;
	y2 = y1;
	z2 = -x1*sB + z1*cB;
	
	// rotate about Z-axis
	x3 = x2*cC - y2*sC;
	y3 = x2*sC + y2*cC;
	z3 = z2;
	
	// return the random rotated vector of magnitude 1.0
	*dx = x3;
	*dy = y3;
	*dz = z3;
	
	return;
}






__declspec(dllexport)

void __cdecl MB_CheckForObjWallCollisions(	unsigned char * bm, const int bmW, const int bmH,
											double * testPtListX, double * testPtListZ,
											double * X0, double * Z0,
											double * Dx, double * Dz, double * Len,
											const int nTestPts, int * colVal,
											double * x2, double * z2 )
{
	
	unsigned char b;
	int i, j, L, x, z;
	double dx, dz, x0, z0, cx, cz;
	
	b = 0;
	*colVal = 0;

	L = (int)(2.0*(*Len) + 1.0);
	dx = *Dx / 2.0;
	dz = *Dz / 2.0;
	x0 = *X0;
	z0 = *Z0;
	
	for (j=0; j<=L; j++){
		cx = x0 + j*dx;
		cz = z0 + j*dz;
		for (i=0; i<nTestPts; i++){
				x = (int)(cx + testPtListX[i]);
				z = (int)(cz + testPtListZ[i]);
				if (x >= 0 && x < bmW && z >= 0 && z < bmH){ 
					b = bm[bmW*z + x];
					if (b != 0){
						*colVal = (int)b;
						*x2 = cx - 2*dx;
						*z2 = cz - 2*dz;
						break;
					}
				}

		}
		if (b){ break; }
	}
	
	return;
	
}






__declspec(dllexport)

void __cdecl MB_CheckForObjWallCollisions2(	unsigned char * bm, const int bmW, const int bmH,
											double * testPtListX, double * testPtListZ,
											double * X0, double * Z0,
											double * Dx, double * Dz, double * Len,
											const int nTestPts, int * nColPts, int * colPtTbl,
											double * x2, double * z2 )
{
	
	unsigned char b;
	int i, j, L, x, z, n;
	double dx, dz, x0, z0, cx, cz;
	
	b = 0;
	n = 0;
	*nColPts = 0;

	L = (int)(2.0*(*Len) + 1.0);
	dx = *Dx / 2.0;
	dz = *Dz / 2.0;
	x0 = *X0;
	z0 = *Z0;
	
	for (j=0; j<=L; j++){
		cx = x0 + j*dx;
		cz = z0 + j*dz;
		for (i=0; i<nTestPts; i++){
				x = (int)(cx + testPtListX[i]);
				z = (int)(cz + testPtListZ[i]);
				if (x >= 0 && x < bmW && z >= 0 && z < bmH){ 
					b = bm[bmW*z + x];
					if (b != 0){
						//*colVal = (int)b;
						colPtTbl[n] = (int)b;
						n++;
						*x2 = cx - 2.5*dx;
						*z2 = cz - 2.5*dz;
						// break;
					}
				}

		}
		if (n > 0){ break; }
	}
	
	*nColPts = n;
	
	return;
	
}



void main(){ return; }

